@@ -14,10 +14,17 @@
 #define __LT_UTILS_H__
 
 #include <stdarg.h>
+#if HAVE_MEMORY_H
+#include <memory.h>
+#endif
+#if HAVE_STRING_H
+#include <string.h>
+#endif
 #include "lt-macros.h"
 
 LT_BEGIN_DECLS
 
+char *lt_getenv        (const char *name);
 int   lt_strcmp0       (const char *v1,
                         const char *v2);
 int   lt_strcasecmp    (const char *s1,
@@ -29,7 +36,9 @@ char *lt_strlower      (char       *string);
 char *lt_strdup_printf (const char *format,
 			...) LT_GNUC_PRINTF (1, 2);
 char *lt_strdup_vprintf(const char *format,
-                        va_list     args);
+                        va_list     args) LT_GNUC_PRINTF (1, 0);
+char *lt_strndup       (const char *s,
+			size_t      n);
 
 LT_END_DECLS
 
